// Copyright (C) 2018-2021 Intel Corporation
// SPDX-License-Identifier: Apache-2.0
//

#pragma once

#include <transformations_visibility.hpp>

#include "ngraph/op/op.hpp"
#include "ngraph/op/constant.hpp"

namespace ngraph {
namespace snippets {
namespace op {

/**
 * @interface Scalar
 * @brief Generated by Canonicalization for a scalar constant Shape() == {1}
 * @ingroup snippets
 */
class TRANSFORMATIONS_API Scalar  : public ngraph::op::Constant {
public:
    OPENVINO_OP("Scalar", "SnippetsOpset", ngraph::op::Constant);

    Scalar() = default;
    Scalar(const std::shared_ptr<runtime::Tensor>& tensor) : Constant(tensor) {}
    template <typename T>
    Scalar(const element::Type& type, Shape shape, const std::vector<T>& values) : Constant(type, shape, values) {}
    Scalar(const element::Type& type, const Shape& shape) : Constant(type, shape) {}
    template <class T, class = typename std::enable_if<std::is_fundamental<T>::value>::type>
    Scalar(const element::Type& type, Shape shape, T value) : Constant(type, shape, value) {}
    Scalar(const element::Type& type, Shape shape, const std::vector<std::string>& values) : Constant(type, shape, values) {}
    Scalar(const element::Type& type, const Shape& shape, const void* data) : Constant(type, shape, data) {}

    Scalar(const Constant& other) : Constant(other) {}
    Scalar(const Scalar& other) : Constant(other) {}
    Scalar& operator=(const Scalar&) = delete;
    ~Scalar() override {}

    std::shared_ptr<Node> clone_with_new_inputs(const OutputVector& new_args) const override {
        check_new_args_count(this, new_args);
        return std::make_shared<Scalar>(*this);
    }
};

} // namespace op
} // namespace snippets
} // namespace ngraph